#! /usr/bin/env bash
#
# Copyright 2021-2022 Xilinx, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
readonly L_CURLY='{'
readonly R_CURLY='}'

Warn () {
    echo WARNING: $ME: $*
}

Die () {
    echo $*
    echo "Usage: $ME DESIGN [BUILD_SUFFIX]"
    echo "    Builds all apps against design for both a72 and r5 processors"
    echo "    Appends -BUILD_SUFFIX, if specified, to build directory name"
    exit 1
}

#
# Input: a name
# Input: a value
# Input: Default location
# Output: sets variables NAME_DIR and NAME, guided by default
ResolvePaths () {
    local name="$1"
    local val="$2"
    local loc="$3"
    local maybe_dir
    local str

    loc=${loc%/}
    val=${val%/}
    if [ -d ${val} ] ; then
        maybe_dir="${val}"
    else
        maybe_dir="${loc}/${val}"
    fi

    if [ -d ${maybe_dir} ]; then
        cat <<EOF >${TMP_ME}
            ${name}_DIR=\$(cd ${maybe_dir}; pwd)
            ${name}=\${${name}_DIR##*/}
EOF
        source ${TMP_ME}
        rm -f ${TMP_ME}
    else
        Die "Error: Directory ${maybe_dir} does not exist"
    fi
}

# Cf. https://stackoverflow.com/questions/9792702/does-bash-support-word-boundary-regular-expressions
DecodeProcCore () {
    local proc=$1
    local core=$2

    if [[ ${proc} = "a72" ]]; then
        #PROC_TUNE="-mcpu=cortex-a72 -mtune=cortex-a72 -march=armv8-a+nofp+nosimd -mgeneral-regs-only"
        #PROC_TUNE="-march=armv8-a+nofp+nosimd -fno-builtin"
        PROC_TUNE="${PROC_TUNE_A72}"
        PROC_BIF_DOMAIN=fpd
        PROC_BIF_EXTRA="
            trustzone
            exception_level = el-3"
        PROC_COMPILER_PREFIX="aarch64-none-elf-"
    elif [[ ${proc} = "r5" ]]; then
        #PROC_TUNE="-mcpu=cortex-r5 -mtune=cortex-r5 -march=armv7-r+nofp+idiv -mgeneral-regs-only"
        #PROC_TUNE="-march=armv7-r+nofp+idiv -fno-builtin"
        PROC_TUNE="${PROC_TUNE_R5}"
        PROC_BIF_DOMAIN=lpd
        PROC_BIF_EXTRA=
        PROC_COMPILER_PREFIX="armr5-none-eabi-"
        if [[ ${core} = "lockstep" ]]; then
            core=0
        fi
    else
        Die "INTERNAL: Invalid processor \'${proc}\'"
    fi
    if [[ ${PROC_TUNE} =~ -fno-builtin ]] ; then
        PROC_TUNE="${PROC_TUNE} -DNO_BUILTIN"
    fi
    eval "PROC_FULL=\${DESIGN_FULLPROC_${proc^^}_${core}}"
    PROC_DOMAIN="DOMAIN_${proc^^}_${core}"
    eval "PROC_DEBUG=\${USER_DEBUG_${core}}"
    eval "PROC_CONSOLE=\${DESIGN_CONSOLE_${core}}"
}

ParseArgs () {
    if [ $# -eq 1 ] ; then
        BUILDDIR_BASE=build
        BOARDTEST_SUFFIX=
    elif [ $# -eq 2 ] ; then
        BUILD_SUFFIX="$2"
        if [[ $BUILD_SUFFIX =~ ^- ]]; then
            BUILDDIR_BASE="build${BUILD_SUFFIX}"
            BOARDTEST_SUFFIX="-${BUILD_SUFFIX}"
        elif [[ $BUILD_SUFFIX =~ ^build- ]]; then
            BUILDDIR_BASE="${BUILD_SUFFIX}"
            BOARDTEST_SUFFIX="$(echo ${BUILD_SUFFIX}|sed -e 's/^build//')"
        else
            BUILDDIR_BASE="build-${BUILD_SUFFIX}"
            BOARDTEST_SUFFIX="-${BUILD_SUFFIX}"
        fi
    else
        Die "Incorrect number of arguments"
    fi

    ResolvePaths DESIGN "$1" "${TOP}/designs"

    DESIGN_PROC="$2"
}

ReadUserConfig () {
    local f="./userconfig.sh"

    if [ ! -f ${f} ]; then
        Die "${ME}: ${f} not found"
    fi

    R5_MINI_APP=
    PROC_TUNE_A72=
    PROC_TUNE_R5=

    source ${f}

    if [ -z "$USER_VIVADO_INSTALL" ]; then
        Die "${ME}: USER_VIVADO_INSTALL not defined in ${f}"
    fi
    if [ -z "$USER_CONSOLE_0" ]; then
        USER_CONSOLE_0=none
    fi
    if [ -z "$USER_TRACE_0" ]; then
        USER_TRACE_0=none
    fi
    if [ -z "$USER_DEBUG_0" ]; then
        USER_DEBUG_0="-O3 -g0"
    fi
    if [ -z "$USER_CONSOLE_1" ]; then
        USER_CONSOLE_1=none
    fi
    if [ -z "$USER_TRACE_1" ]; then
        USER_TRACE_1=none
    fi
    if [ -z "$USER_DEBUG_1" ]; then
        USER_DEBUG_1="-O3 -g0"
    fi

    if [ "${USER_CONSOLE_0}" != none ]; then
        UART_FLAGS_0="-DUART_CONTROLLER"
        UART_FLAGS_1=""
    elif [ "${USER_CONSOLE_1}" != none ]; then
        UART_FLAGS_0=""
        UART_FLAGS_1="-DUART_CONTROLLER"
    else
        UART_FLAGS_0=""
        UART_FLAGS_1=""
    fi
}

PrepareBoardTestDir () {
    rm -rf ${DESIGN_BOARDTEST_DIR}
    mkdir ${DESIGN_BOARDTEST_DIR}
    mkdir -p ${DESIGN_BOARDTEST_DIR}/images

    cp ${DESIGN_PDI} ${DESIGN_BOARDTEST_DIR}
    mkdir ${DESIGN_BOARDTEST_DIR}/srcs
    tar -C .. -c -f - x86_app | tar -C ${DESIGN_BOARDTEST_DIR}/srcs -x -v -f -
    tar -C .. -c -f - common | tar -C ${DESIGN_BOARDTEST_DIR}/srcs -x -v -f -
    cp ../x86_app/x86_app ${DESIGN_BOARDTEST_DIR}
    cp ../x86_app/boards.csv ${DESIGN_BOARDTEST_DIR}
    cp ../x86_app/images.csv ${DESIGN_BOARDTEST_DIR}
    cp -r ../deployment_scripts ${DESIGN_BOARDTEST_DIR}
    cp -r ../pan_uio ${DESIGN_BOARDTEST_DIR}
}

DecodeDesign () {
    local proc_tune
    local tcl_cmd
    local tcl_output
    local xsa
    local proc
    local core
    local rtc
    local pat_cons_0
    local pat_cons_1
    local pat_r5_0
    local pat_r5_1
    local pat_a72_0
    local pat_a72_1
    local pat_rtc_0

    pushd "${DESIGN_DIR}"
        DESIGN_BUILD_DIR="${DESIGN_DIR}/${BUILDDIR_BASE}"
        DESIGN_XSCT="${DESIGN_BUILD_DIR}/build.xsct"
        DESIGN_WORKSPACE="${DESIGN_BUILD_DIR}/WORKSPACE"
        DESIGN_PLATFORM_NAME="PLATFORM"
        DESIGN_BOARDTEST_DIR="${DESIGN_BUILD_DIR}/boardtest_${DESIGN}${BOARDTEST_SUFFIX}"

        x=$(ls *.xsa|wc -l)
        if [ $x -ne 1 ] ; then
            Die $DESIGN_DIR: Need single XSA file in directory, counted "'$x'"
        fi
        DESIGN_XSA="$(echo *.xsa)"
        DESIGN_XSA="${DESIGN_DIR}/${DESIGN_XSA}"

        x=$(ls *.pdi|egrep -v '_presynth\.pdi' |wc -l)
        if [ $x -ne 1 ] ; then
            Die $DESIGN_DIR: Need single Vivado PDI file in directory, counted "'$x'"
        fi
        DESIGN_PDI="$(ls *.pdi|egrep -v '_presynth\.pdi')"
        DESIGN_PDI="${DESIGN_DIR}/${DESIGN_PDI}"

        DESIGN_COMPILE_FLAGS="-fno-strict-aliasing -ffunction-sections -fdata-sections -fno-tree-loop-distribute-patterns"
        DESIGN_LINK_FLAGS="${DESIGN_COMPILE_FLAGS} -nolibc -Wl,--gc-sections"

        rm -rf "${DESIGN_BUILD_DIR}"
        mkdir "${DESIGN_BUILD_DIR}"
        pushd "${DESIGN_BUILD_DIR}"
            ln -s $(realpath --canonicalize-missing --relative-to=. ${DESIGN_XSA}) .
            xsa=$(echo *.xsa)

            # e.g. design_procs=(versal_cips_0_pspmc_0_psv_cortexa72_0 ...)
            tcl_cmd="
            setws . ;
            set procs [getprocessors $xsa];
            foreach {key} \$procs {puts proc=\$key};
            "

            if [ "${USER_CONSOLE_0}" = "none" ] ; then
                DESIGN_CONSOLE_0="none"
            else
                DESIGN_CONSOLE_0=
                tcl_cmd="${tcl_cmd} ;
                set console0 [lsearch -inline -regexp [getperipherals $xsa] {$USER_CONSOLE_0}];
                puts console0=\${console0};
                "
            fi

            if [ "${USER_CONSOLE_1}" = "none" ] ; then
                DESIGN_CONSOLE_1="none"
            else
                DESIGN_CONSOLE_1=
                tcl_cmd="${tcl_cmd} ;
                set console1 [lsearch -inline -regexp [getperipherals $xsa] {$USER_CONSOLE_1}];
                puts console1=\${console1};
                "
            fi

            tcl_cmd="${tcl_cmd} ;
            set rtc [lsearch -inline -regexp [getperipherals $xsa] {rtc_0}];
            puts rtc=\${rtc};
            "

            tcl_cmd="${tcl_cmd};exit"
            readarray -t tcl_output < <(echo ${tcl_cmd}|xsct -quiet -no-ini 2>/dev/null|egrep -v INFO)
        popd

        DESIGN_FULLPROC_R5_0=
        DESIGN_FULLPROC_R5_1=
        DESIGN_FULLPROC_A72_0=
        DESIGN_FULLPROC_A72_1=
        DESIGN_RTC_0=
        pat_cons_0="^console0=.*$"
        pat_cons_1="^console1=.*$"
        pat_r5_0="^proc=.*r5_0$"
        pat_r5_1="^proc=.*r5_1$"
        pat_a72_0="^proc=.*a72_0$"
        pat_a72_1="^proc=.*a72_1$"
        pat_rtc_0="^rtc=.*$"
        for i in "${tcl_output[@]}"; do
            echo "Considering: $i"
            if [[ $i =~ $pat_r5_0 ]]; then
                DESIGN_FULLPROC_R5_0=$(echo $i|sed -e 's/^proc=//')
            elif [[ $i =~ $pat_r5_1 ]]; then
                DESIGN_FULLPROC_R5_1=$(echo $i|sed -e 's/^proc=//')
            elif [[ $i =~ $pat_a72_0 ]]; then
                DESIGN_FULLPROC_A72_0=$(echo $i|sed -e 's/^proc=//')
            elif [[ $i =~ $pat_a72_1 ]]; then
                DESIGN_FULLPROC_A72_1=$(echo $i|sed -e 's/^proc=//')
            elif [[ $i =~ $pat_cons_0 ]]; then
                DESIGN_CONSOLE_0=$(echo $i|sed -e 's/^console0=//')
            elif [[ $i =~ $pat_cons_1 ]]; then
                DESIGN_CONSOLE_1=$(echo $i|sed -e 's/^console1=//')
            elif [[ $i =~ $pat_rtc_0 ]]; then
                DESIGN_RTC_0=$(echo $i|sed -e 's/^rtc=//')
            fi
        done
        if [[ "X$DESIGN_FULLPROC_R5_0" = "X" ]]; then
            Die "Cannot find processor $pat_r5_0 in $DESIGN_XSA"
        fi
        if [[ "X$DESIGN_FULLPROC_R5_1" = "X" ]]; then
            Die "Cannot find processor $pat_r5_1 in $DESIGN_XSA"
        fi
        if [[ "X$DESIGN_FULLPROC_A72_0" = "X" ]]; then
            Die "Cannot find processor $pat_a72_0 in $DESIGN_XSA"
        fi
        if [[ "X$DESIGN_FULLPROC_A72_1" = "X" ]]; then
            Die "Cannot find processor $pat_a72_1 in $DESIGN_XSA"
        fi

        if [ "X${DESIGN_CONSOLE_0}" = "X" ]; then
            Die "No console found for ${USER_CONSOLE_0}"
        fi

        if [ "X${DESIGN_CONSOLE_1}" = "X" ]; then
            Die "No console found for ${USER_CONSOLE_1}"
        fi
        if [ "X${DESIGN_RTC_0}" = "X" ]; then
            RTC_DRIVER="#No RTC in design, skip setdriver"
        else
            RTC_DRIVER="bsp setdriver -ip ${DESIGN_RTC_0} -driver none"
        fi
    popd

    touch ${DESIGN_XSCT}

# Default bsp extra compiler flags: -g -Wall -Wextra -Dversal -DARMA72_EL3 -fno-tree-loop-distribute-patterns
    cat <<EOF >>${DESIGN_XSCT}
setws ${DESIGN_WORKSPACE}
platform create -name PLATFORM -hw ${DESIGN_XSA##*/}

EOF

    for proc in a72 r5; do
        for core in 0 1; do
            DecodeProcCore $proc $core

            cat <<EOF1 >>${DESIGN_XSCT}
domain create -name ${PROC_DOMAIN} -proc ${PROC_FULL} -os standalone
bsp config -append extra_compiler_flags "${DESIGN_COMPILE_FLAGS} ${PROC_DEBUG} ${PROC_TUNE}"
bsp config stdin "${PROC_CONSOLE}"
bsp config stdout "${PROC_CONSOLE}"

${RTC_DRIVER}
EOF1
        done
    done
}

DecodeApp () {
    local app_name="$1"
    local proc="$2"
    local core="$3"
    local str

    # Sets APP, APP_DIR
    ResolvePaths APP "$app_name" "${TOP}"
    if [[ ${proc} = "r5" && ${core} = "lockstep" ]]; then
        core=0
    fi

    cat <<EOF >${TMP_ME}
        APP_NAME="\${APP}_SYSPROJ${SYSPROJ_INDEX}_${proc}_${core}"

        case \${USER_TRACE_${core}} in
        console)
            if [ "\${PROC_CONSOLE}" != "none" ]; then
                TRACE_FLAGS='-DDO_TRACE'
            fi
            ;;
        coresight)
            TRACE_FLAGS='-DDO_TRACE -DDO_TRACE_CORESIGHT'
            ;;
        *)
            TRACE_FLAGS=
            ;;
        esac
        APP_DEBUG_FLAGS="\${USER_DEBUG_${core}} \${TRACE_FLAGS} \${UART_FLAGS_${core}}"
EOF
    source ${TMP_ME}
    rm -f ${TMP_ME}
    if [ -f ${APP_DIR}/do.sh ]; then
        APP_ELF="${APP_DIR}/${APP_NAME}.elf"
    else
        APP_ELF="${DESIGN_WORKSPACE}/${APP_NAME}/Release/${APP_NAME}.elf"
    fi
}

AddFile () {
    local f=$1
    local lof

    lof=$(readlink -f ${f})
    if [[ "$lof" =~ \.ld$ ]] ; then
        echo "importsources -name ${APP_NAME} -linker-script -path ${lof}" >> ${DESIGN_XSCT}
    elif [ "X${lof}" != "X" ]; then
        echo "importsources -name ${APP_NAME} -path ${lof}" >> ${DESIGN_XSCT}
    fi
}

ScanAndAddFiles () {
    local dir="$1"
    local proc="$2"
    local core="$3"
    local f
    local g
    local proc_core_dir
    
    if [[ ${proc} = "r5" && ${core} = "lockstep" ]]; then
        core=0
    fi
    proc_core_dir="${proc}_${core}"

    for f in $(ls ${dir}); do
        if [ -f ${dir}/${f} ]; then
            if [[ ${f} == *.ld ]]; then
                continue
            fi
            AddFile ${dir}/${f}
        else
            for g in $(ls ${dir}/${f}); do
                if [[ ${g} == *.ld ]]; then
                    continue
                elif [[ ${f} == ${proc} && -f ${dir}/${f}/${g} ]] ; then
                    AddFile ${dir}/${f}/${g}
                fi
            done
        fi
    done
}

FindAndAddLinkerScript () {
    local proc=$1
    local core=$2
    local path

    if [[ ${proc} = "r5" && ${core} = "lockstep" ]]; then
        core=0
    fi
    for path in ${APP_DIR}/${proc}_${core} ${TOP}/agent_common/${proc}_${core}; do
        if [ -d $path ]; then
            for f in $(ls ${path}); do
                if [[ -f "${path}/$f" && $f =~ ^.*\.ld$ ]] ; then
                    AddFile ${path}/${f}
                    return
                fi
            done
        fi
    done

    Die "No linker script found for app ${APP_NAME} ${proc}_${core}"
}

AddApp () {
    local sysproj="$1"
    local app="$2"
    local proc="$3"
    local core="$4"
    local f
    local g
    local app_elf

    DecodeProcCore $proc $core

    # Resolves APP, APP_DIR, APP_NAME, APP_DEBUG_FLAGS, APP_ELF
    DecodeApp ${app} ${proc} ${core}

    if [ -f ${APP_DIR}/do.sh ]; then
        pushd ${APP_DIR}
            source ./do.sh
        popd
    else
        cat <<EOF >>${DESIGN_XSCT}
app create -name ${APP_NAME} -platform PLATFORM -domain ${PROC_DOMAIN} -sysproj ${sysproj} -template "Empty Application(C)"
app config -name ${APP_NAME} build-config release
app config -name $APP_NAME -add compiler-misc "${DESIGN_COMPILE_FLAGS} ${APP_DEBUG_FLAGS} -D${proc}_${core} ${PROC_TUNE}"
app config -name $APP_NAME -add linker-misc "${DESIGN_COMPILE_FLAGS} ${APP_DEBUG_FLAGS} ${DESIGN_LINK_FLAGS} ${PROC_TUNE}"
EOF

        FindAndAddLinkerScript ${proc} ${core}

        ScanAndAddFiles ${APP_DIR} ${proc} ${core}
        ScanAndAddFiles ${TOP}/common ${proc} ${core}
        ScanAndAddFiles ${TOP}/agent_common ${proc} ${core}

        echo "" >>${DESIGN_XSCT}
    fi

    app_elf=$(basename ${APP_ELF})
    pushd ${SYSPROJ_BOOT_DIR}
        # Note: The elf does not exist at this point but a symbolic link can be placed ahead of time
        ln -s $(realpath --canonicalize-missing --relative-to=. ${APP_ELF}) ${app_elf}
    popd

    cat <<EOF2 >>${SYSPROJ_BIF}
    image
    ${L_CURLY}
        name = ${PROC_BIF_DOMAIN}
        partition
        ${L_CURLY}
            core = ${proc}-${core}${PROC_BIF_EXTRA}
            file = ${app_elf}
        ${R_CURLY}
    ${R_CURLY}
EOF2

    cat <<EOF3 >>${SYSPROJ_DO}
${PROC_COMPILER_PREFIX}objdump -D -S -x -s ${APP_NAME}.elf > ${APP_NAME}.objdump.txt

${PROC_COMPILER_PREFIX}nm -n -a ${APP_NAME}.elf > ${APP_NAME}.nm.txt

EOF3
}

AddSysprojPreludes () {
    local pdi

    rm -f ${SYSPROJ_BIF}

    pushd ${SYSPROJ_BOOT_DIR}
        pdi=$(basename ${DESIGN_PDI})
        ln -s $(realpath --canonicalize-missing --relative-to=. ${DESIGN_PDI}) ${pdi}
    popd

    cat << EOF >>${SYSPROJ_BIF}
new_bif:
${L_CURLY}
    image
    ${L_CURLY}
        ${L_CURLY}
            type = bootimage
            file = ${pdi}
        ${R_CURLY}
    ${R_CURLY}
EOF
        cat <<EOF2 >>${SYSPROJ_DO}
#! /usr/bin/env bash
set -x
set -e
bootgen \
        -arch versal \
        -fill 0xff \
        -image ${SYSPROJ_BIF} \
        -log trace \
        -w on \
        -o "${SYSPROJ_BIN}"
cp ${SYSPROJ_BIN} ${DESIGN_BOARDTEST_DIR}/images
(echo -n "$(basename ${SYSPROJ_BIN}),prom,"; hexdump -n 64 -e '4/4 "%08X"' /dev/urandom; echo "") \
    >> ${DESIGN_BOARDTEST_DIR}/images.csv
EOF2
}

PackageBoardTest () {
    f="${DESIGN_BOARDTEST_DIR}/deployment_scripts/do.sh"
    cat <<EOF >"${f}"
#! /usr/bin/env bash
# Only root should execute this script
if [[ \$EUID -ne 0 ]]; then
    echo "\$0: Must be run as root"
    exit 1
fi

set -x
set -e
cat boards.csv|egrep -v '^#'
lspci -vvv|egrep -C 6 Xilinx
echo Use the Device command to choose the appropriate PCIe device
./x86_app -p images -i images.csv -b boards.csv -x -v 
EOF
    chmod 755 "${f}"

    tar \
        -C $(dirname ${DESIGN_BOARDTEST_DIR}) \
        -c \
        -v \
        -j \
        -f ${DESIGN_BOARDTEST_DIR}.tgj \
        $(basename ${DESIGN_BOARDTEST_DIR})
    rm -rf ${DESIGN_BOARDTEST_DIR}
}

AddSysprojPostludes () {
    echo "${R_CURLY}" >>${SYSPROJ_BIF}
    pushd ${SYSPROJ_BOOT_DIR}
    popd
}

AddSysproj () {
    local proc=$1
    local app0=$2
    local app1=$3
    local sysproj

    sysproj="SYSPROJ${SYSPROJ_INDEX}_${proc}_${app0}_${app1}"

    SYSPROJ_BOOT_DIR="${DESIGN_BUILD_DIR}/boot_${sysproj}"
    rm -rf ${SYSPROJ_BOOT_DIR}
    mkdir ${SYSPROJ_BOOT_DIR}

    SYSPROJ_BIF="${SYSPROJ_BOOT_DIR}/my.bif"
    touch ${SYSPROJ_BIF}

    SYSPROJ_BIN="${SYSPROJ_BOOT_DIR}/${sysproj}.bin"
    SYSPROJ_DO="${SYSPROJ_BOOT_DIR}/do.sh"
    touch ${SYSPROJ_DO}
    chmod 755 ${SYSPROJ_DO}

    AddSysprojPreludes
    if [ ! -z "${R5_MINI_APP}" ]; then
        if [ $proc = "a72" ]; then
            AddApp $sysproj "${R5_MINI_APP}" r5 lockstep
        fi
    fi
    AddApp $sysproj $app0 $proc 0
    AddApp $sysproj $app1 $proc 1
    AddSysprojPostludes

    SYSPROJ_INDEX=$((SYSPROJ_INDEX+1))
}

set -x
set -e
ME=$0
TMP_ME="${ME}_tmp.sh"
readonly TOP=$(cd ..;pwd)

ParseArgs $*

ReadUserConfig

pushd ${TOP}/x86_app
    ./clean.sh
    ./do.sh
popd
pushd ${TOP}/sim_images
    ./clean.sh
    ./do.sh
popd

source ${USER_VIVADO_INSTALL}/settings64.sh

DecodeDesign "${DESIGN}"
PrepareBoardTestDir

SYSPROJ_INDEX=0
for proc in a72 r5; do
    AddSysproj $proc hello_app hello_app
    AddSysproj $proc prog_app rem_app
    AddSysproj $proc prog_multiboot_app rem_app
    AddSysproj $proc multiboot_app rem_app
done

pushd "${DESIGN_BUILD_DIR}"
    echo "platform generate" >>${DESIGN_XSCT}
    echo "app build -all" >>${DESIGN_XSCT}
    xsct -quiet -no-ini "${DESIGN_XSCT}" 2>&1 |tee xsct.out
popd

for d in ${DESIGN_BUILD_DIR}/boot_* ; do
    pushd $d
        ./do.sh
    popd
done

PackageBoardTest

exit 0
